@@ -69,6 +69,10 @@ public:
 
   using base_type::iterator;
   using base_type::reverse_iterator;
+  using base_type::const_iterator;
+
+  using base_type::begin;
+  using base_type::end;
   using base_type::size;
   using base_type::empty;
 
