@@ -36,8 +36,6 @@
 
 #include "config.h"
 
-#include <algorithm>
-
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/mman.h>
@@ -149,14 +147,4 @@ MemoryChunk::sync(uint32_t offset, uint32_t length, int flags) {
   return msync(m_ptr + offset, length, flags) == 0;
 }    
 
-bool
-MemoryChunk::is_incore(uint32_t offset, uint32_t length) {
-  uint32_t size = pages_touched(offset, length);
-  char buf[size];
-  
-  incore(buf, offset, length);
-
-  return std::find(buf, buf + size, 0) == buf + size;
-}
-
 }
