@@ -70,7 +70,7 @@ AvailableList::push_back(const rak::socket_address* sa) {
 
 void
 AvailableList::insert(AddressList* l) {
-  if (size() > m_maxSize)
+  if (!want_more())
     return;
 
   std::sort(begin(), end());
@@ -90,8 +90,12 @@ void
 AvailableList::erase(const rak::socket_address& sa) {
   iterator itr = std::find(begin(), end(), sa);
 
-  if (itr != end())
-    erase(itr);
+  if (itr != end()) {
+    value_type tmp = *itr;
+    *itr = back();
+
+    pop_back();
+  }
 }
 
 }
