@@ -97,9 +97,13 @@ ChunkSelector::find(PeerChunks* pc, __UNUSED bool highPriority) {
   // set.
   rak::partial_queue* queue = pc->is_seeder() ? &m_sharedQueue : pc->download_cache();
 
-  // DEBUG: Remove this, just for testing syncing.
-//   m_position = random() % size();
-//   queue->clear();
+  // Randomize position on average every 16 chunks to prevent
+  // inefficient distribution with a slow seed and fast peers
+  // all arriving at the same position.
+  if ((random() & 63) == 0) {
+    m_position = random() % size();
+    queue->clear();
+  }
 
   if (queue->is_enabled()) {
 
