@@ -43,12 +43,12 @@
 
 #include "globals.h"
 
-#include "download_info.h"
 #include "delegator.h"
 
 #include "data/chunk_handle.h"
 #include "download/available_list.h"
 #include "net/data_buffer.h"
+#include "torrent/download_info.h"
 #include "torrent/data/file_list.h"
 #include "torrent/peer/peer_list.h"
 
@@ -116,6 +116,8 @@ public:
 
   bool                want_pex_msg()                             { return m_info->is_pex_active() && m_peerList.available_list()->want_more(); }; 
 
+  void                set_metadata_size(size_t s);
+
   // Carefull with these.
   void                setup_delegator();
   void                setup_tracker();
@@ -146,6 +148,9 @@ public:
 
   void                update_endgame();
 
+  rak::priority_item& delay_disconnect_peers() { return m_delayDisconnectPeers; }
+  rak::priority_item& delay_download_done() { return m_delayDownloadDone; }
+
 private:
   // Disable copy ctor and assignment.
   DownloadMain(const DownloadMain&);
@@ -187,6 +192,8 @@ private:
   SlotCountHandshakes m_slotCountHandshakes;
   SlotHashCheckAdd    m_slotHashCheckAdd;
 
+  rak::priority_item  m_delayDownloadDone;
+  rak::priority_item  m_delayDisconnectPeers;
   rak::priority_item  m_taskTrackerRequest;
 };
 
