@@ -36,10 +36,10 @@
 
 #include "config.h"
 
-#include "download/download_info.h"
 #include "download/download_main.h"
 #include "net/throttle_list.h"
 #include "torrent/dht_manager.h"
+#include "torrent/download_info.h"
 #include "torrent/exceptions.h"
 #include "torrent/error.h"
 #include "torrent/poll.h"
@@ -605,6 +605,33 @@ Handshake::read_extension() {
   return true;
 }
 
+bool
+Handshake::read_port() {
+  if (m_readBuffer.peek_32() > m_readBuffer.reserved())
+    throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_value);
+
+  int32_t need = m_readBuffer.peek_32() + 4 - m_readBuffer.remaining();
+
+  if (need + 5 > m_readBuffer.reserved_left()) {
+    m_readBuffer.move_unused();
+
+    if (need + 5 > m_readBuffer.reserved_left())
+      throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_value);
+  }
+
+  if (!fill_read_buffer(m_readBuffer.peek_32() + 4))
+    return false;
+
+  uint32_t length = m_readBuffer.read_32() - 1;
+  m_readBuffer.read_8();
+
+  if (length == 2)
+    manager->dht_manager()->add_node(m_address.c_sockaddr(), m_readBuffer.peek_16());
+
+  m_readBuffer.consume(length);
+  return true;
+}
+
 void
 Handshake::read_done() {
   if (m_readDone != false)
@@ -723,6 +750,17 @@ restart:
 
     case READ_MESSAGE:
     case POST_HANDSHAKE:
+      // For meta-downloads, we aren't interested in the bitfield or
+      // extension messages here, PCMetadata handles all that. The
+      // bitfield only refers to the single-chunk meta-data, so fake that.
+      if (m_download->info()->is_meta_download()) {
+        m_bitfield.set_size_bits(1);
+        m_bitfield.allocate();
+        m_bitfield.set(0);
+        read_done();
+        break;
+      }
+
       fill_read_buffer(5);
 
       // Received a keep-alive message which means we won't be
@@ -764,6 +802,12 @@ restart:
         m_readPos = 0;
         m_state = READ_EXT;
 
+      } else if (m_readBuffer.peek_8_at(4) == protocol_port) {
+        // Some peers seem to send the port message before handshake,
+        // so handle it here.
+        m_readPos = 0;
+        m_state = READ_PORT;
+
       } else {
         read_done();
         break;
@@ -771,10 +815,12 @@ restart:
 
     case READ_BITFIELD:
     case READ_EXT:
+    case READ_PORT:
       // Gather the different command types into the same case group
       // so that we don't need 'goto restart' above.
       if ((m_state == READ_BITFIELD && !read_bitfield()) ||
-          (m_state == READ_EXT && !read_extension()))
+          (m_state == READ_EXT && !read_extension()) ||
+          (m_state == READ_PORT && !read_port()))
         break;
 
       m_state = READ_MESSAGE;
@@ -1022,6 +1068,10 @@ Handshake::prepare_peer_info() {
   std::memcpy(m_peerInfo->set_options(), m_options, 8);
   m_peerInfo->mutable_id().assign((const char*)m_readBuffer.position());
   m_readBuffer.consume(20);
+
+  // For meta downloads, we require support of the extension protocol.
+  if (m_download->info()->is_meta_download() && !m_peerInfo->supports_extensions())
+    throw handshake_error(ConnectionManager::handshake_dropped, e_handshake_unwanted_connection);
 }
 
 void
