@@ -37,7 +37,9 @@
 #include "config.h"
 
 #include <cstdio>
+#include <fcntl.h>
 #include <rak/error_number.h>
+#include <rak/string_manip.h>
 
 #include "torrent/exceptions.h"
 #include "torrent/data/block.h"
@@ -47,10 +49,10 @@
 #include "download/choke_manager.h"
 #include "download/chunk_selector.h"
 #include "download/chunk_statistics.h"
-#include "download/download_info.h"
 #include "download/download_main.h"
 #include "net/socket_base.h"
 #include "torrent/connection_manager.h"
+#include "torrent/download_info.h"
 #include "torrent/throttle.h"
 #include "torrent/peer/peer_info.h"
 #include "torrent/peer/connection_list.h"
@@ -79,7 +81,9 @@ PeerConnectionBase::PeerConnectionBase() :
   m_sendPEXMask(0),
 
   m_encryptBuffer(NULL),
-  m_extensions(NULL) {
+  m_extensions(NULL),
+
+  m_incoreContinous(false) {
 
   m_peerInfo = NULL;
 }
@@ -93,8 +97,7 @@ PeerConnectionBase::~PeerConnectionBase() {
   if (m_extensions != NULL && !m_extensions->is_default())
     delete m_extensions;
 
-  if (m_extensionMessage.owned())
-    m_extensionMessage.clear();
+  m_extensionMessage.clear();
 }
 
 void
@@ -116,6 +119,8 @@ PeerConnectionBase::initialize(DownloadMain* download, PeerInfo* peerInfo, Socke
   m_encryption = *encryptionInfo;
   m_extensions = extensions;
 
+  m_extensions->set_connection(this);
+
   m_peerChunks.set_peer_info(m_peerInfo);
   m_peerChunks.bitfield()->swap(*bitfield);
 
@@ -275,11 +280,63 @@ PeerConnectionBase::receive_download_choke(bool choke) {
   return true;
 }
 
+inline static void
+log_upload_chunk_mincore(Chunk* chunk, const Piece& piece, bool new_index, bool& continous) {
+#ifdef LT_LOG_MINCORE_FILE
+  static int mincore_fd = -1;
+  static int32_t ticker = rak::timer::current().seconds() / 10 * 10;
+
+  static int counter_incore = 0;
+  static int counter_not_incore = 0;
+  static int counter_incore_new = 0;
+  static int counter_not_incore_new = 0;
+  static int counter_incore_break = 0;
+
+  if (rak::timer::current().seconds() >= ticker + 10) {
+    char buffer[256];
+
+    if (mincore_fd == -1) {
+      snprintf(buffer, 256, LT_LOG_MINCORE_FILE, getpid());
+    
+      if ((mincore_fd = open(buffer, O_WRONLY | O_CREAT | O_TRUNC)) == -1)
+        throw internal_error("Could not open mincore log file.");
+    }
+
+    // Log the result of mincore for every piece uploaded to a file.
+    unsigned int buf_lenght = snprintf(buffer, 256, "%i %u %u %u %u %u\n",
+                                       ticker,
+                                       counter_incore, counter_incore_new, counter_not_incore,
+                                       counter_not_incore_new, counter_incore_break);
+
+    write(mincore_fd, buffer, buf_lenght);
+    
+    ticker = rak::timer::current().seconds() / 10 * 10;
+
+    counter_incore = 0;
+    counter_not_incore = 0;
+    counter_incore_new = 0;
+    counter_not_incore_new = 0;
+    counter_incore_break = 0;
+  }
+
+  bool is_incore = chunk->is_incore(piece.offset(), piece.length());
+
+  counter_incore += !new_index && is_incore;
+  counter_incore_new += new_index && is_incore;
+  counter_not_incore += !new_index && !is_incore;
+  counter_not_incore_new += new_index && !is_incore;
+
+  counter_incore_break += continous && !is_incore;
+  continous = is_incore;
+#endif
+}
+
 void
 PeerConnectionBase::load_up_chunk() {
   if (m_upChunk.is_valid() && m_upChunk.index() == m_upPiece.index()) {
     // Better checking needed.
     //     m_upChunk.chunk()->preload(m_upPiece.offset(), m_upChunk.chunk()->size());
+    log_upload_chunk_mincore(m_upChunk.chunk(), m_upPiece, false, m_incoreContinous);
     return;
   }
 
@@ -295,6 +352,10 @@ PeerConnectionBase::load_up_chunk() {
     m_encryptBuffer->reset();
   }
 
+  m_incoreContinous = false;
+  log_upload_chunk_mincore(m_upChunk.chunk(), m_upPiece, true, m_incoreContinous);
+  m_incoreContinous = true;
+
   // Also check if we've already preloaded in the recent past, even
   // past unmaps.
   ChunkManager* cm = manager->chunk_manager();
@@ -426,7 +487,7 @@ PeerConnectionBase::down_chunk() {
   transfer->adjust_position(bytesTransfered);
 
   m_down->throttle()->node_used(m_peerChunks.download_throttle(), bytesTransfered);
-  m_download->info()->down_rate()->insert(bytesTransfered);
+  m_download->info()->mutable_down_rate()->insert(bytesTransfered);
 
   return transfer->is_finished();
 }
@@ -496,7 +557,7 @@ PeerConnectionBase::down_chunk_process(const void* buffer, uint32_t length) {
   transfer->adjust_position(length);
 
   m_down->throttle()->node_used(m_peerChunks.download_throttle(), length);
-  m_download->info()->down_rate()->insert(length);
+  m_download->info()->mutable_down_rate()->insert(length);
 
   return length;
 }
@@ -515,8 +576,8 @@ PeerConnectionBase::down_chunk_skip_process(const void* buffer, uint32_t length)
   // Hmm, this might result in more bytes than nessesary being
   // counted.
   m_down->throttle()->node_used(m_peerChunks.download_throttle(), length);
-  m_download->info()->down_rate()->insert(length);
-  m_download->info()->skip_rate()->insert(length);
+  m_download->info()->mutable_down_rate()->insert(length);
+  m_download->info()->mutable_skip_rate()->insert(length);
 
   if (!transfer->is_valid()) {
     transfer->adjust_position(length);
@@ -581,8 +642,12 @@ PeerConnectionBase::down_extension() {
     m_extensions->read_move(bytes);
   }
 
-  if (m_extensions->is_complete())
-    m_extensions->read_done();
+  // If extension can't be processed yet (due to a pending write),
+  // disable reads until the pending message is completely sent.
+  if (m_extensions->is_complete() && !m_extensions->is_invalid() && !m_extensions->read_done()) {
+    manager->poll()->remove_read(this);
+    return false;
+  }
 
   return m_extensions->is_complete();
 }
@@ -657,7 +722,7 @@ PeerConnectionBase::up_chunk() {
   }
 
   m_up->throttle()->node_used(m_peerChunks.upload_throttle(), bytesTransfered);
-  m_download->info()->up_rate()->insert(bytesTransfered);
+  m_download->info()->mutable_up_rate()->insert(bytesTransfered);
 
   // Just modifying the piece to cover the remaining data ends up
   // being much cleaner and we avoid an unnessesary position variable.
@@ -693,12 +758,16 @@ PeerConnectionBase::up_extension() {
   if (m_extensionOffset < m_extensionMessage.length())
     return false;
 
-  // clear() deletes the buffer, only do that if we made a copy,
-  // otherwise the buffer is shared among all connections.
-  if (m_extensionMessage.owned())
-    m_extensionMessage.clear();
-  else 
-    m_extensionMessage.set(NULL, NULL, false);
+  m_extensionMessage.clear();
+
+  // If we have an unprocessed message, process it now and enable reads again.
+  if (m_extensions->is_complete() && !m_extensions->is_invalid()) {
+    // DEBUG: What, this should fail when we block, no?
+    if (!m_extensions->read_done())
+      throw internal_error("PeerConnectionBase::up_extension could not process complete extension message.");
+
+    manager->poll()->insert_read(this);
+  }
 
   return true;
 }
@@ -857,4 +926,16 @@ PeerConnectionBase::send_pex_message() {
   return true;
 }
 
+// Extension protocol needs to send a reply.
+bool
+PeerConnectionBase::send_ext_message() {
+  write_prepare_extension(m_extensions->pending_message_type(), m_extensions->pending_message_data());
+  m_extensions->clear_pending_message();
+  return true;
+}
+
+void
+PeerConnectionBase::receive_metadata_piece(uint32_t piece, const char* data, uint32_t length) {
+}
+
 }
