@@ -258,11 +258,12 @@ PeerList::disconnected(PeerInfo* p, int flags) {
   
   iterator itr = std::find_if(range.first, range.second, rak::equal(p, rak::mem_ref(&value_type::second)));
 
-  if (itr == range.second)
+  if (itr == range.second) {
     if (std::find_if(base_type::begin(), base_type::end(), rak::equal(p, rak::mem_ref(&value_type::second))) == base_type::end())
       throw internal_error("PeerList::disconnected(...) itr == range.second, doesn't exist.");
     else
       throw internal_error("PeerList::disconnected(...) itr == range.second, not in the range.");
+  }
   
   disconnected(itr, flags);
 }
