@@ -46,6 +46,7 @@
 #include <torrent/event.h>
 
 #include "poll_kqueue.h"
+#include "thread_base.h"
 
 #ifdef USE_KQUEUE
 #include <sys/types.h>
@@ -212,6 +213,9 @@ PollKQueue::perform() {
     if (itr->ident < 0 || itr->ident >= m_table.size())
       continue;
 
+    if ((flags() & flag_waive_global_lock) && ThreadBase::global_queue_size() != 0)
+      ThreadBase::waive_global_lock();
+
     Table::iterator evItr = m_table.begin() + itr->ident;
 
     if ((itr->flags & EV_ERROR) && evItr->second != NULL) {
